' Palette fading routines by Petter Holmberg, -97:

' --- Define palette storage arrays:

TYPE PalType
  red AS INTEGER
  green AS INTEGER
  blue AS INTEGER
END TYPE

DIM rgbpal(0 TO 255) AS PalType
DIM palmem(0 TO 255) AS PalType

' --- Declare SUBs:

DECLARE SUB WaitRetrace ()

DECLARE SUB FadeOut (keepcolor%, rangestart%, rangeend%, rgbpal() AS PalType, palmem() AS PalType)
DECLARE SUB FadeIn (keepcolor%, rangestart%, rangeend%, rgbpal() AS PalType, palmem() AS PalType)
DECLARE SUB BrightenOut (keepcolor%, rangestart%, rangeend%, rgbpal() AS PalType, palmem() AS PalType)
DECLARE SUB BrightenIn (keepcolor%, rangestart%, rangeend%, rgbpal() AS PalType, palmem() AS PalType)

' --- Print start text:

CLS
SCREEN 0
WIDTH 80, 25

PRINT "Palette fading routines by Petter Holmberg, -97:"
PRINT
PRINT "This program demonstrates four different palette fading routines."
PRINT "First a picture will be shown on the screen. Then it will fade out to black,"
PRINT "using the SUB FadeOut. Then it will fade back to its previous colors with the"
PRINT "SUB FadeIn. Then it will be brightened out to white, using the SUB BrightenOut."
PRINT "Finally, it will be faded back with the SUB BrightenIn."
PRINT
PRINT "A few words should be said about this demo..."
PRINT "Note that the colors in the SUBs FadeIn and BrightenIn all reach their final"
PRINT "Intensity at the same time. Many fading routines doesn't do that. Also note"
PRINT "that I don't write to port 3C7h/3C8h more than once when reading/writing a"
PRINT "whole range of palette colors, since the VGA hardware auto-increases the color"
PRINT "index sent to these ports after reading/writing port 3C9h three times. This is"
PRINT "a VGA secret many people don't know, and it also makes palette routines a lot"
PRINT "faster and reduces flicker. Experiment with the variable keepcolor% used in all"
PRINT "of the fading SUBs and you will see some cool color effects."
PRINT
PRINT "You may use these fading routines in your own programs as long as you credit"
PRINT "me in your code. Petter Holmberg, -97"

DO: LOOP WHILE INKEY$ = ""

' --- Set the screen mode and load example picture:

SCREEN 13

DEF SEG = &HA000
BLOAD "EXAMPLE.GRH", 0
DEF SEG

' --- Demonstrate fading effects:

SLEEP 2
FadeOut 0, 0, 255, rgbpal(), palmem()
SLEEP 2
FadeIn 0, 0, 255, rgbpal(), palmem()
SLEEP 2
BrightenOut 0, 0, 255, rgbpal(), palmem()
SLEEP 2
BrightenIn 0, 0, 255, rgbpal(), palmem()
SLEEP 2

' --- End the program:

SCREEN 0
WIDTH 80, 25

PRINT "That's all for now..."
PRINT "Petter Holmberg, -97"

SUB BrightenIn (keepcolor%, rangestart%, rangeend%, rgbpal() AS PalType, palmem() AS PalType)

' Description:
' ------------
' This SUB fades back a previously brightened out screen:
'
' Input:
' ------
' palmem() : Previous palette values.
'
' keepcolor%  : Color to keep (0:none, 1:red, 2:green, 3:blue,
'                              4:yellow, 5:magneta, 6:cyan, 7:all)
' rangestart% : First color index to brighten in (0-255).
' rangeend%   : Last color index to brighten in (0-255).

' --- Save the present palette for modification:

OUT &H3C7, rangestart%

FOR savepresentpal% = rangestart% TO rangeend%
   rgbpal(savepresentpal%).red = INP(&H3C9)
   rgbpal(savepresentpal%).green = INP(&H3C9)
   rgbpal(savepresentpal%).blue = INP(&H3C9)
NEXT savepresentpal%

' --- Set the new palette:

FOR brighteninpal% = 0 TO 63

   FOR brightenscan% = rangestart% TO rangeend%
      SELECT CASE keepcolor%
      CASE 0
       IF palmem(brightenscan%).red < brighteninpal% AND rgbpal(brightenscan%).red > palmem(brightenscan%).red THEN rgbpal(brightenscan%).red = rgbpal(brightenscan%).red - 1
       IF palmem(brightenscan%).green < brighteninpal% AND rgbpal(brightenscan%).green > palmem(brightenscan%).green THEN rgbpal(brightenscan%).green = rgbpal(brightenscan%).green - 1
       IF palmem(brightenscan%).blue < brighteninpal% AND rgbpal(brightenscan%).blue > palmem(brightenscan%).blue THEN rgbpal(brightenscan%).blue = rgbpal(brightenscan%).blue - 1
      CASE 1
       IF palmem(brightenscan%).green < brighteninpal% AND rgbpal(brightenscan%).green > palmem(brightenscan%).green THEN rgbpal(brightenscan%).green = rgbpal(brightenscan%).green - 1
       IF palmem(brightenscan%).blue < brighteninpal% AND rgbpal(brightenscan%).blue > palmem(brightenscan%).blue THEN rgbpal(brightenscan%).blue = rgbpal(brightenscan%).blue - 1
      CASE 2
       IF palmem(brightenscan%).red < brighteninpal% AND rgbpal(brightenscan%).red > palmem(brightenscan%).red THEN rgbpal(brightenscan%).red = rgbpal(brightenscan%).red - 1
       IF palmem(brightenscan%).blue < brighteninpal% AND rgbpal(brightenscan%).blue > palmem(brightenscan%).blue THEN rgbpal(brightenscan%).blue = rgbpal(brightenscan%).blue - 1
      CASE 3
       IF palmem(brightenscan%).red < brighteninpal% AND rgbpal(brightenscan%).red > palmem(brightenscan%).red THEN rgbpal(brightenscan%).red = rgbpal(brightenscan%).red - 1
       IF palmem(brightenscan%).green < brighteninpal% AND rgbpal(brightenscan%).green > palmem(brightenscan%).green THEN rgbpal(brightenscan%).green = rgbpal(brightenscan%).green - 1
      CASE 4
       IF palmem(brightenscan%).blue < brighteninpal% AND rgbpal(brightenscan%).blue > palmem(brightenscan%).blue THEN rgbpal(brightenscan%).blue = rgbpal(brightenscan%).blue - 1
      CASE 5
       IF palmem(brightenscan%).green < brighteninpal% AND rgbpal(brightenscan%).green > palmem(brightenscan%).green THEN rgbpal(brightenscan%).green = rgbpal(brightenscan%).green - 1
      CASE 6
       IF palmem(brightenscan%).red < brighteninpal% AND rgbpal(brightenscan%).red > palmem(brightenscan%).red THEN rgbpal(brightenscan%).red = rgbpal(brightenscan%).red - 1
      END SELECT
   NEXT brightenscan%

   OUT &H3C8, rangestart%
   WaitRetrace

   FOR brightenpal% = rangestart% TO rangeend%
      OUT &H3C9, rgbpal(brightenpal%).red
      OUT &H3C9, rgbpal(brightenpal%).green
      OUT &H3C9, rgbpal(brightenpal%).blue
   NEXT brightenpal%

NEXT brighteninpal%

END SUB

SUB BrightenOut (keepcolor%, rangestart%, rangeend%, rgbpal() AS PalType, palmem() AS PalType)

' Description:
' ------------
' This SUB brightens desired RGB values to white:
'
' Input:
' ------
' keepcolor%  : Color to keep (0:none, 1:red, 2:green, 3:blue,
'                              4:yellow, 5:magneta, 6:cyan, 7:all)
' rangestart% : First color index to brighten out (0-255).
' rangeend%   : Last color index to brighten out (0-255).
'
' Output:
' ------
' palmem() : Previous palette values.

' --- Save the old palette:

OUT &H3C7, rangestart%

FOR saveoldpal% = rangestart% TO rangeend%
   palmem(saveoldpal%).red = INP(&H3C9)
   palmem(saveoldpal%).green = INP(&H3C9)
   palmem(saveoldpal%).blue = INP(&H3C9)
NEXT saveoldpal%

' --- Save the present palette for modification:

OUT &H3C7, rangestart%

FOR savepresentpal% = rangestart% TO rangeend%
   rgbpal(savepresentpal%).red = INP(&H3C9)
   rgbpal(savepresentpal%).green = INP(&H3C9)
   rgbpal(savepresentpal%).blue = INP(&H3C9)
NEXT savepresentpal%

' --- Set the new palette:

FOR brightenoutpal% = 0 TO 63
   FOR brightenscan% = rangestart% TO rangeend%
      SELECT CASE keepcolor%
      CASE 0
       IF rgbpal(brightenscan%).red < 63 THEN rgbpal(brightenscan%).red = rgbpal(brightenscan%).red + 1
       IF rgbpal(brightenscan%).green < 63 THEN rgbpal(brightenscan%).green = rgbpal(brightenscan%).green + 1
       IF rgbpal(brightenscan%).blue < 63 THEN rgbpal(brightenscan%).blue = rgbpal(brightenscan%).blue + 1
      CASE 1
       IF rgbpal(brightenscan%).green < 63 THEN rgbpal(brightenscan%).green = rgbpal(brightenscan%).green + 1
       IF rgbpal(brightenscan%).blue < 63 THEN rgbpal(brightenscan%).blue = rgbpal(brightenscan%).blue + 1
      CASE 2
       IF rgbpal(brightenscan%).red < 63 THEN rgbpal(brightenscan%).red = rgbpal(brightenscan%).red + 1
       IF rgbpal(brightenscan%).blue < 63 THEN rgbpal(brightenscan%).blue = rgbpal(brightenscan%).blue + 1
      CASE 3
       IF rgbpal(brightenscan%).red < 63 THEN rgbpal(brightenscan%).red = rgbpal(brightenscan%).red + 1
       IF rgbpal(brightenscan%).green < 63 THEN rgbpal(brightenscan%).green = rgbpal(brightenscan%).green + 1
      CASE 4
       IF rgbpal(brightenscan%).blue < 63 THEN rgbpal(brightenscan%).blue = rgbpal(brightenscan%).blue + 1
      CASE 5
       IF rgbpal(brightenscan%).green < 63 THEN rgbpal(brightenscan%).green = rgbpal(brightenscan%).green + 1
      CASE 6
       IF rgbpal(brightenscan%).red < 63 THEN rgbpal(brightenscan%).red = rgbpal(brightenscan%).red + 1
      END SELECT
   NEXT brightenscan%

   OUT &H3C8, rangestart%
   WaitRetrace

   FOR brightenpal% = rangestart% TO rangeend%
      OUT &H3C9, rgbpal(brightenpal%).red
      OUT &H3C9, rgbpal(brightenpal%).green
      OUT &H3C9, rgbpal(brightenpal%).blue
   NEXT brightenpal%

NEXT brightenoutpal%

END SUB

SUB FadeIn (keepcolor%, rangestart%, rangeend%, rgbpal() AS PalType, palmem() AS PalType)

' Description:
' ------------
' This SUB fades back a previously faded out screen:
'
' Input:
' ------
' palmem() : Previous palette values.
'
' keepcolor%  : Color to keep (0:none, 1:red, 2:green, 3:blue,
'                              4:yellow, 5:magneta, 6:cyan, 7:all)
' rangestart% : First color index to fade in (0-255).
' rangeend%   : Last color index to fade in (0-255).

' --- Save the present palette for modification:

OUT &H3C7, rangestart%

FOR savepresentpal% = rangestart% TO rangeend%
   rgbpal(savepresentpal%).red = INP(&H3C9)
   rgbpal(savepresentpal%).green = INP(&H3C9)
   rgbpal(savepresentpal%).blue = INP(&H3C9)
NEXT savepresentpal%

' --- Set the new palette:

FOR fadeinpal% = 0 TO 63

   FOR fadescan% = rangestart% TO rangeend%
      SELECT CASE keepcolor%
      CASE 0
       IF 63 - palmem(fadescan%).red < fadeinpal% AND rgbpal(fadescan%).red < palmem(fadescan%).red THEN rgbpal(fadescan%).red = rgbpal(fadescan%).red + 1
       IF 63 - palmem(fadescan%).green < fadeinpal% AND rgbpal(fadescan%).green < palmem(fadescan%).green THEN rgbpal(fadescan%).green = rgbpal(fadescan%).green + 1
       IF 63 - palmem(fadescan%).blue < fadeinpal% AND rgbpal(fadescan%).blue < palmem(fadescan%).blue THEN rgbpal(fadescan%).blue = rgbpal(fadescan%).blue + 1
      CASE 1
       IF 63 - palmem(fadescan%).green < fadeinpal% AND rgbpal(fadescan%).green < palmem(fadescan%).green THEN rgbpal(fadescan%).green = rgbpal(fadescan%).green + 1
       IF 63 - palmem(fadescan%).blue < fadeinpal% AND rgbpal(fadescan%).blue < palmem(fadescan%).blue THEN rgbpal(fadescan%).blue = rgbpal(fadescan%).blue + 1
      CASE 2
       IF 63 - palmem(fadescan%).red < fadeinpal% AND rgbpal(fadescan%).red < palmem(fadescan%).red THEN rgbpal(fadescan%).red = rgbpal(fadescan%).red + 1
       IF 63 - palmem(fadescan%).blue < fadeinpal% AND rgbpal(fadescan%).blue < palmem(fadescan%).blue THEN rgbpal(fadescan%).blue = rgbpal(fadescan%).blue + 1
      CASE 3
       IF 63 - palmem(fadescan%).red < fadeinpal% AND rgbpal(fadescan%).red < palmem(fadescan%).red THEN rgbpal(fadescan%).red = rgbpal(fadescan%).red + 1
       IF 63 - palmem(fadescan%).green < fadeinpal% AND rgbpal(fadescan%).green < palmem(fadescan%).green THEN rgbpal(fadescan%).green = rgbpal(fadescan%).green + 1
      CASE 4
       IF 63 - palmem(fadescan%).blue < fadeinpal% AND rgbpal(fadescan%).blue < palmem(fadescan%).blue THEN rgbpal(fadescan%).blue = rgbpal(fadescan%).blue + 1
      CASE 5
       IF 63 - palmem(fadescan%).green < fadeinpal% AND rgbpal(fadescan%).green < palmem(fadescan%).green THEN rgbpal(fadescan%).green = rgbpal(fadescan%).green + 1
      CASE 6
       IF 63 - palmem(fadescan%).red < fadeinpal% AND rgbpal(fadescan%).red < palmem(fadescan%).red THEN rgbpal(fadescan%).red = rgbpal(fadescan%).red + 1
      END SELECT
   NEXT fadescan%

   OUT &H3C8, rangestart%
   WaitRetrace

   FOR fadepal% = rangestart% TO rangeend%
      OUT &H3C9, rgbpal(fadepal%).red
      OUT &H3C9, rgbpal(fadepal%).green
      OUT &H3C9, rgbpal(fadepal%).blue
   NEXT fadepal%

NEXT fadeinpal%

END SUB

SUB FadeOut (keepcolor%, rangestart%, rangeend%, rgbpal() AS PalType, palmem() AS PalType)

' Description:
' ------------
' This SUB fades desired RGB values to black:
'
' Input:
' ------
' keepcolor%  : Color to keep (0:none, 1:red, 2:green, 3:blue,
'                              4:yellow, 5:magneta, 6:cyan, 7:all)
' rangestart% : First color index to fade out (0-255).
' rangeend%   : Last color index to fade out (0-255).
'
' Output:
' ------
' palmem() : Previous palette values.

' --- Save the old palette:

OUT &H3C7, rangestart%

FOR saveoldpal% = rangestart% TO rangeend%
   palmem(saveoldpal%).red = INP(&H3C9)
   palmem(saveoldpal%).green = INP(&H3C9)
   palmem(saveoldpal%).blue = INP(&H3C9)
NEXT saveoldpal%

' --- Save the present palette for modification:

OUT &H3C7, rangestart%

FOR savepresentpal% = rangestart% TO rangeend%
   rgbpal(savepresentpal%).red = INP(&H3C9)
   rgbpal(savepresentpal%).green = INP(&H3C9)
   rgbpal(savepresentpal%).blue = INP(&H3C9)
NEXT savepresentpal%

' --- Set the new palette:

FOR fadeoutpal% = 0 TO 63
   FOR fadescan% = rangestart% TO rangeend%
      SELECT CASE keepcolor%
      CASE 0
       IF rgbpal(fadescan%).red > 0 THEN rgbpal(fadescan%).red = rgbpal(fadescan%).red - 1
       IF rgbpal(fadescan%).green > 0 THEN rgbpal(fadescan%).green = rgbpal(fadescan%).green - 1
       IF rgbpal(fadescan%).blue > 0 THEN rgbpal(fadescan%).blue = rgbpal(fadescan%).blue - 1
      CASE 1
       IF rgbpal(fadescan%).green > 0 THEN rgbpal(fadescan%).green = rgbpal(fadescan%).green - 1
       IF rgbpal(fadescan%).blue > 0 THEN rgbpal(fadescan%).blue = rgbpal(fadescan%).blue - 1
      CASE 2
       IF rgbpal(fadescan%).red > 0 THEN rgbpal(fadescan%).red = rgbpal(fadescan%).red - 1
       IF rgbpal(fadescan%).blue > 0 THEN rgbpal(fadescan%).blue = rgbpal(fadescan%).blue - 1
      CASE 3
       IF rgbpal(fadescan%).red > 0 THEN rgbpal(fadescan%).red = rgbpal(fadescan%).red - 1
       IF rgbpal(fadescan%).green > 0 THEN rgbpal(fadescan%).green = rgbpal(fadescan%).green - 1
      CASE 4
       IF rgbpal(fadescan%).blue > 0 THEN rgbpal(fadescan%).blue = rgbpal(fadescan%).blue - 1
      CASE 5
       IF rgbpal(fadescan%).green > 0 THEN rgbpal(fadescan%).green = rgbpal(fadescan%).green - 1
      CASE 6
       IF rgbpal(fadescan%).red > 0 THEN rgbpal(fadescan%).red = rgbpal(fadescan%).red - 1
      END SELECT
   NEXT fadescan%

   OUT &H3C8, rangestart%
   WaitRetrace

   FOR fadepal% = rangestart% TO rangeend%
      OUT &H3C9, rgbpal(fadepal%).red
      OUT &H3C9, rgbpal(fadepal%).green
      OUT &H3C9, rgbpal(fadepal%).blue
   NEXT fadepal%

NEXT fadeoutpal%

END SUB

SUB WaitRetrace

' Description:
' ------------
' This SUB waits for the beginning of a vertical retrace.
'
' Usage:
' ------
' This SUB eliminates screen flickering during palette modification.

WAIT (&H3DA), 8    ' Wait for a screen update.
WAIT (&H3DA), 8, 8 ' Wait for the vertical retrace.

END SUB

